@@ -3,24 +3,26 @@
 /**************************************************************/
 
 #include "lighting_ui.h"
+#include "lighting_pixmaps.h"
 
 extern LightingValues mapvals;
 
 GckApplicationWindow *appwin            = NULL;
 GtkWidget            *color_select_diag = NULL;
-GckNoteBook          *options_notebook  = NULL;
-GckNoteBookPage      *bump_page         = NULL;
-GckNoteBookPage      *env_page          = NULL;
-GckNoteBookPage      *ref_page          = NULL;
+GtkNotebook          *options_note_book = NULL;
+GtkWidget            *bump_page         = NULL;
+GtkWidget            *env_page          = NULL;
 GtkTooltips          *tooltips          = NULL;
 GdkGC                *gc                = NULL;
 
 GtkWidget *previewarea,*pointlightwid,*dirlightwid;
-GtkWidget *imagewid,*waveswid;
+GtkWidget *imagewid,*waveswid,*bump_label,*env_label;
+GtkWidget *xentry,*yentry,*zentry;
 
 GckRGB old_light_color;
 
 gint color_dialog_id = -1;
+gint bump_page_pos,env_page_pos;
 
 guint left_button_pressed = FALSE, light_hit = FALSE;
 guint32 blackpixel,whitepixel;
@@ -82,7 +84,10 @@ void exit_callback            (GtkWidget *widget, gpointer client_data);
 void color_ok_callback        (GtkWidget *widget, gpointer client_data);
 void color_changed_callback   (GtkColorSelection *colorsel, gpointer client_data);
 void color_cancel_callback    (GtkWidget *widget, gpointer client_data);
-void lightcolor_callback      (GtkWidget *widget, gpointer client_data);
+void light_color_callback     (GtkWidget *widget, gpointer client_data);
+
+GtkWidget *create_bump_page        (void);
+GtkWidget *create_environment_page (void);
 
 gint bumpmap_constrain         (gint32 image_id, gint32 drawable_id, gpointer data);
 void bumpmap_drawable_callback (gint32 id, gpointer data);
@@ -169,6 +174,18 @@ void togglequality_update(GtkWidget *button, gpointer data)
   draw_preview_image(TRUE);
 }
 
+/**********************************/
+/* Toggle refractive layer update */
+/**********************************/
+
+void togglerefraction_update(GtkWidget *button, gpointer data)
+{
+  gint *value;
+
+  value=(gint *)gtk_object_get_data(GTK_OBJECT(button),"ValuePtr");
+  *value=!(*value);
+}
+
 /*****************************/
 /* Toggle bumpmapping update */
 /*****************************/
@@ -181,21 +198,20 @@ void togglebump_update(GtkWidget *button, gpointer data)
   *value=!(*value);
   
   if (mapvals.bump_mapped==TRUE)
-    gck_notebook_append_page(options_notebook,bump_page);
+    {
+      bump_page_pos = g_list_length(options_note_book->children);
+
+      bump_page = create_bump_page();
+      bump_label = gtk_label_new("Bump");
+      gtk_notebook_append_page(options_note_book,bump_page,bump_label);
+    }
   else
-    gck_notebook_remove_page(options_notebook,bump_page->position);
-}
-
-/**********************************/
-/* Toggle refractive layer update */
-/**********************************/
-
-void togglerefraction_update(GtkWidget *button, gpointer data)
-{
-  gint *value;
-
-  value=(gint *)gtk_object_get_data(GTK_OBJECT(button),"ValuePtr");
-  *value=!(*value);
+    {     
+      gtk_notebook_remove_page(options_note_book,bump_page_pos);
+      if (bump_page_pos<env_page_pos)
+        env_page_pos--;
+      bump_page_pos = 0;
+    }
 }
 
 /*************************************/
@@ -210,9 +226,21 @@ void toggleenvironment_update(GtkWidget *button, gpointer data)
   *value=!(*value);
 
   if (mapvals.env_mapped==TRUE)
-    gck_notebook_append_page(options_notebook,env_page);
+    {
+      env_page_pos = g_list_length(options_note_book->children);
+
+      env_page = create_environment_page();
+      env_label = gtk_label_new("Env");
+
+      gtk_notebook_append_page(options_note_book,env_page,env_label);
+    }
   else
-    gck_notebook_remove_page(options_notebook,env_page->position);
+    {
+      gtk_notebook_remove_page(options_note_book,env_page_pos);
+      if (env_page_pos<bump_page_pos)
+        bump_page_pos--;
+      env_page_pos = 0;
+    }
 }
 
 /******************************/
@@ -352,6 +380,8 @@ void apply_callback(GtkWidget *widget, gpointer client_data)
   if (image!=NULL)
     gdk_image_destroy(image);
 
+  gtk_object_unref(GTK_OBJECT(tooltips));
+  
   gck_application_window_destroy(appwin);
   gdk_flush();
 
@@ -374,7 +404,9 @@ void exit_callback(GtkWidget *widget, gpointer client_data)
 
   if (backbuf.image!=NULL)
     gdk_image_destroy(backbuf.image);
-
+  
+  gtk_object_unref(GTK_OBJECT(tooltips));
+  
   gck_application_window_destroy(appwin);
   
   gtk_main_quit();
@@ -411,7 +443,7 @@ void color_cancel_callback(GtkWidget *widget, gpointer client_data)
   color_select_diag=NULL;
 }
 
-void lightcolor_callback(GtkWidget *widget, gpointer client_data)
+void light_color_callback(GtkWidget *widget, gpointer client_data)
 {
   GtkColorSelectionDialog *csd;
 
@@ -509,21 +541,19 @@ gint preview_events(GtkWidget *area, GdkEvent  *event)
 /* Dialog constructors */
 /***********************/
 
-void create_main_notebook(GtkWidget *container)
+GtkWidget *create_options_page(void)
 {
-  GtkWidget *frame,*vbox,*vbox2,*toggleanti,*togglebump,*togglerefraction;
-  GtkWidget *toggletrans,*toggleimage,*toggletips,*togglequality,*hbox;
-  GtkWidget *widget1,*widget2,*widget3,*toggleenvironment;
-  GckNoteBookPage *page1,*page2,*page3;
-
-  options_notebook=gck_notebook_new(container,300,320,GTK_POS_TOP);
+  GtkWidget *page,*frame,*toggletrans,*toggleimage;
+  GtkWidget *toggletips,*togglequality,*vbox,*hbox;
+  GtkWidget *toggleenvironment,*toggleanti,*togglebump;
+  GtkWidget *widget1,*widget2,*label;
 
   /* General options page */
   /* ==================== */
 
-  page1=gck_notebook_page_new("Options",options_notebook);
+  page = gck_vbox_new(NULL,FALSE,FALSE,FALSE,0,0,0);
 
-  frame=gck_frame_new("General options",page1->widget,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
+  frame=gck_frame_new("General options",page,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
   vbox=gck_vbox_new(frame,FALSE,FALSE,FALSE,0,5,5);
 
   togglebump=gck_checkbutton_new("Use bump mapping",vbox,mapvals.bump_mapped,
@@ -541,13 +571,13 @@ void create_main_notebook(GtkWidget *container)
   toggletips=gck_checkbutton_new("Enable tooltips",vbox,mapvals.tooltips_enabled,
     (GtkSignalFunc)toggletips_update);
 
-  gtk_tooltips_set_tip (tooltips,togglebump,"Enable/disable bump-mapping (image depth)",NULL);
-  gtk_tooltips_set_tip (tooltips,toggleenvironment,"Enable/disable environment mapping (reflection)",NULL);
-/*  gtk_tooltips_set_tip (tooltips,togglerefraction,"Enable/disable refractive layer",NULL); */
-  gtk_tooltips_set_tip (tooltips,toggletrans,"Make destination image transparent where bump height is zero",NULL);
-  gtk_tooltips_set_tip (tooltips,toggleimage,"Create a new image when applying filter",NULL);
-  gtk_tooltips_set_tip (tooltips,togglequality,"Enable/disable high quality previews",NULL);
-  gtk_tooltips_set_tip (tooltips,toggletips,"Enable/disable tooltip messages",NULL); 
+  gtk_tooltips_set_tip(tooltips,togglebump,"Enable/disable bump-mapping (image depth)",NULL);
+  gtk_tooltips_set_tip(tooltips,toggleenvironment,"Enable/disable environment mapping (reflection)",NULL);
+/*  gtk_tooltips_set_tips(tooltips,togglerefraction,"Enable/disable refractive layer"); */
+  gtk_tooltips_set_tip(tooltips,toggletrans,"Make destination image transparent where bump height is zero",NULL);
+  gtk_tooltips_set_tip(tooltips,toggleimage,"Create a new image when applying filter",NULL);
+  gtk_tooltips_set_tip(tooltips,togglequality,"Enable/disable high quality previews",NULL);
+  gtk_tooltips_set_tip(tooltips,toggletips,"Enable/disable tooltip messages",NULL); 
 
   gtk_object_set_data(GTK_OBJECT(togglebump),"ValuePtr",(gpointer)&mapvals.bump_mapped);
   gtk_object_set_data(GTK_OBJECT(toggleenvironment),"ValuePtr",(gpointer)&mapvals.env_mapped);
@@ -557,20 +587,41 @@ void create_main_notebook(GtkWidget *container)
   gtk_object_set_data(GTK_OBJECT(togglequality),"ValuePtr",(gpointer)&mapvals.previewquality);
   gtk_object_set_data(GTK_OBJECT(toggletips),"ValuePtr", (gpointer)&mapvals.tooltips_enabled);
 
-  frame=gck_frame_new("Antialiasing options",page1->widget,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
+  gtk_widget_show(togglebump);
+  gtk_widget_show(toggleenvironment);
+  gtk_widget_show(toggletrans);
+  gtk_widget_show(toggleimage);
+  gtk_widget_show(togglequality);
+  gtk_widget_show(toggletips);
+  gtk_widget_show(vbox);
+  gtk_widget_show(frame);
+
+  frame=gck_frame_new("Antialiasing options",page,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
   vbox=gck_vbox_new(frame,FALSE,FALSE,FALSE,5,5,5);
 
   toggleanti=gck_checkbutton_new("Enable antialiasing",vbox,mapvals.antialiasing,
     (GtkSignalFunc)toggleanti_update);
   gtk_object_set_data(GTK_OBJECT(toggleanti),"ValuePtr",(gpointer)&mapvals.antialiasing);
-  gtk_tooltips_set_tip (tooltips,toggleanti,"Enable/disable jagged edges removal (antialiasing)",NULL);
+  gtk_tooltips_set_tip(tooltips,toggleanti,"Enable/disable jagged edges removal (antialiasing)",NULL);
    
   hbox=gck_hbox_new(vbox,FALSE,TRUE,TRUE,0,0,0);
+  
+  gtk_widget_show(toggleanti);
+  gtk_widget_show(vbox);
+  gtk_widget_show(frame);
+
   vbox=gck_vbox_new(hbox,TRUE,FALSE,TRUE,0,0,0);
   frame=gck_frame_new(NULL,vbox,GTK_SHADOW_NONE,TRUE,TRUE,0,0);
-  gck_label_aligned_new("Depth:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_BOTTOM);
+  label=gck_label_aligned_new("Depth:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_BOTTOM);
+  gtk_widget_show(label);
+  gtk_widget_show(frame);
+  
   frame=gck_frame_new(NULL,vbox,GTK_SHADOW_NONE,TRUE,TRUE,0,0);
-  gck_label_aligned_new("Treshold:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
+  label=gck_label_aligned_new("Treshold:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
+  gtk_widget_show(label);
+  gtk_widget_show(vbox);
+  gtk_widget_show(frame);
+
   vbox=gck_vbox_new(hbox,TRUE,FALSE,FALSE,5,0,0);
 
   widget1=gck_hscale_new(NULL,vbox,&sample_scale_vals,(GtkSignalFunc)scale_update);
@@ -578,127 +629,273 @@ void create_main_notebook(GtkWidget *container)
   gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.max_depth);
   gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.pixel_treshold);
 
-  gtk_tooltips_set_tip (tooltips,widget1,"Antialiasing quality. Higher is better, but slower",NULL);
-  gtk_tooltips_set_tip (tooltips,widget2,"Stop when pixel differences are smaller than this value",NULL);
+  gtk_tooltips_set_tip(tooltips,widget1,"Antialiasing quality. Higher is better, but slower",NULL);
+  gtk_tooltips_set_tip(tooltips,widget2,"Stop when pixel differences are smaller than this value",NULL);
 
-  /* Light settings page */
-  /* =================== */
+  gtk_widget_show(widget1);
+  gtk_widget_show(widget2);
+  gtk_widget_show(vbox);
+  gtk_widget_show(hbox);
 
-  page2=gck_notebook_page_new("Light",options_notebook);
+  gtk_widget_show(page);
+
+  return(page);
+}
+
+/******************************/
+/* Create light settings page */
+/******************************/
+
+GtkWidget *create_light_page(void)
+{
+  GtkWidget *page,*frame,*vbox;
+  GtkWidget *widget1,*widget2,*widget3;
+
+  page=gtk_vbox_new(FALSE,0);
   
-  frame=gck_frame_new("Light settings",page2->widget,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
+  frame=gck_frame_new("Light settings",page,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
   vbox=gck_vbox_new(frame,FALSE,TRUE,TRUE,5,0,5);
 
+  gck_auto_show(TRUE);
   widget1=gck_option_menu_new("Lightsource type:",vbox,TRUE,TRUE,0,
     light_labels,(GtkSignalFunc)lightmenu_callback, NULL);
   gtk_option_menu_set_history(GTK_OPTION_MENU(widget1),mapvals.lightsource.type);
+  gck_auto_show(FALSE);
   
-  widget2=gck_pushbutton_new("Lightsource color",vbox,TRUE,FALSE,0,(GtkSignalFunc)lightcolor_callback);
+  widget2=gck_pushbutton_new("Lightsource color",vbox,TRUE,FALSE,0,
+    (GtkSignalFunc)light_color_callback);
+
+  gtk_widget_show(widget2);
+  gtk_widget_show(vbox);
+  gtk_widget_show(frame);
+
+  gtk_tooltips_set_tip(tooltips,widget1,"Type of light source to apply",NULL);
+  gtk_tooltips_set_tip(tooltips,widget2,"Set light source color (white is default)",NULL);
   
-  gtk_tooltips_set_tip (tooltips,widget1,"Type of light source to apply",NULL);
-  gtk_tooltips_set_tip (tooltips,widget2,"Set light source color (white is default)",NULL);
-  
-  pointlightwid=gck_frame_new("Position",page2->widget,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
+  pointlightwid=gck_frame_new("Position",page,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
   vbox=gck_vbox_new(pointlightwid,FALSE,FALSE,FALSE,5,0,5);
-  widget1=gck_entryfield_new("X:",vbox,mapvals.lightsource.position.x,(GtkSignalFunc)entry_update);
-  widget2=gck_entryfield_new("Y:",vbox,mapvals.lightsource.position.y,(GtkSignalFunc)entry_update);
-  widget3=gck_entryfield_new("Z:",vbox,mapvals.lightsource.position.z,(GtkSignalFunc)entry_update);
-  gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.lightsource.position.x);
-  gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.lightsource.position.y);
-  gtk_object_set_data(GTK_OBJECT(widget3),"ValuePtr",(gpointer)&mapvals.lightsource.position.z);
 
-  gtk_tooltips_set_tip (tooltips,widget1,"Light source X position in XYZ space",NULL);
-  gtk_tooltips_set_tip (tooltips,widget2,"Light source Y position in XYZ space",NULL);
-  gtk_tooltips_set_tip (tooltips,widget3,"Light source Z position in XYZ space",NULL);
+  xentry=gck_entryfield_new("X:",vbox,mapvals.lightsource.position.x,(GtkSignalFunc)entry_update);
+  yentry=gck_entryfield_new("Y:",vbox,mapvals.lightsource.position.y,(GtkSignalFunc)entry_update);
+  zentry=gck_entryfield_new("Z:",vbox,mapvals.lightsource.position.z,(GtkSignalFunc)entry_update);
 
-  dirlightwid=gck_frame_new("Direction vector",page2->widget,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
+  gtk_object_set_data(GTK_OBJECT(xentry),"ValuePtr",(gpointer)&mapvals.lightsource.position.x);
+  gtk_object_set_data(GTK_OBJECT(yentry),"ValuePtr",(gpointer)&mapvals.lightsource.position.y);
+  gtk_object_set_data(GTK_OBJECT(zentry),"ValuePtr",(gpointer)&mapvals.lightsource.position.z);
+
+  gtk_tooltips_set_tip(tooltips,xentry,"Light source X position in XYZ space",NULL);
+  gtk_tooltips_set_tip(tooltips,yentry,"Light source Y position in XYZ space",NULL);
+  gtk_tooltips_set_tip(tooltips,zentry,"Light source Z position in XYZ space",NULL);
+
+  gtk_widget_show(xentry);
+  gtk_widget_show(yentry);
+  gtk_widget_show(zentry);
+  gtk_widget_show(vbox);
+  gtk_widget_show(frame);
+  gtk_widget_show(pointlightwid);
+
+  dirlightwid=gck_frame_new("Direction vector",page,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
   vbox=gck_vbox_new(dirlightwid,FALSE,FALSE,FALSE,5,0,5);
+
   widget1=gck_entryfield_new("X:",vbox,mapvals.lightsource.direction.x,(GtkSignalFunc)entry_update);
   widget2=gck_entryfield_new("Y:",vbox,mapvals.lightsource.direction.y,(GtkSignalFunc)entry_update);
   widget3=gck_entryfield_new("Z:",vbox,mapvals.lightsource.direction.z,(GtkSignalFunc)entry_update);
 
-  gtk_tooltips_set_tip (tooltips,widget1,"Light source X direction in XYZ space",NULL);
-  gtk_tooltips_set_tip (tooltips,widget2,"Light source Y direction in XYZ space",NULL);
-  gtk_tooltips_set_tip (tooltips,widget3,"Light source Z direction in XYZ space",NULL);
+  gtk_tooltips_set_tip(tooltips,widget1,"Light source X direction in XYZ space",NULL);
+  gtk_tooltips_set_tip(tooltips,widget2,"Light source Y direction in XYZ space",NULL);
+  gtk_tooltips_set_tip(tooltips,widget3,"Light source Z direction in XYZ space",NULL);
 
   gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.lightsource.direction.x);
   gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.lightsource.direction.y);
   gtk_object_set_data(GTK_OBJECT(widget3),"ValuePtr",(gpointer)&mapvals.lightsource.direction.z);
-  gtk_widget_hide(dirlightwid);
-  gtk_widget_hide(page2->widget);
 
-  /* Material settings page */
-  /* ====================== */
+  gtk_widget_show(widget1);
+  gtk_widget_show(widget2);
+  gtk_widget_show(widget3);
+  gtk_widget_show(vbox);
 
-  page3=gck_notebook_page_new("Material",options_notebook);
+  gtk_widget_show(page);
 
-  frame=gck_frame_new("Intensity levels",page3->widget,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
-  hbox=gck_hbox_new(frame,FALSE,TRUE,TRUE,0,0,5);
-  vbox=gck_vbox_new(hbox,TRUE,FALSE,TRUE,0,0,0);
-  frame=gck_frame_new(NULL,vbox,GTK_SHADOW_NONE,TRUE,TRUE,0,0);
-  gck_label_aligned_new("Ambient:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
-  frame=gck_frame_new(NULL,vbox,GTK_SHADOW_NONE,TRUE,TRUE,0,0);
-  gck_label_aligned_new("Diffuse:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
-  vbox=gck_vbox_new(hbox,FALSE,FALSE,FALSE,5,0,0);
-  widget1=gck_entryfield_new(NULL,vbox,mapvals.material.ambient_int,(GtkSignalFunc)entry_update);
-  widget2=gck_entryfield_new(NULL,vbox,mapvals.material.diffuse_int,(GtkSignalFunc)entry_update);
+  return page;  
+}
 
-  gtk_tooltips_set_tip (tooltips,widget1,"Amount of original color to show where no direct light falls",NULL);
-  gtk_tooltips_set_tip (tooltips,widget2,"Intensity of original color when lit by a light source",NULL);
+/*********************************/
+/* Create material settings page */
+/*********************************/
+
+GtkWidget *create_material_page(void)
+{
+  GtkWidget *page,*frame,*table;
+  GtkWidget *label1,*label2,*label3;
+  GtkWidget *widget1,*widget2,*widget3;
+  GdkPixmap *image;
+  GdkBitmap *mask;
+  GtkStyle  *style;
+  GtkWidget *pixmap;
+  
+  page=gtk_vbox_new(FALSE,0);
+
+  frame=gck_frame_new("Intensity levels",page,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
+
+  table=gtk_table_new(2,4,FALSE);
+  gtk_container_add(GTK_CONTAINER(frame),table);
+  
+  label1=gck_label_aligned_new("Ambient:",NULL,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
+  label2=gck_label_aligned_new("Diffuse:",NULL,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
+
+  gtk_table_attach(GTK_TABLE(table),label1,0,1,0,1, 0,0,0,0);
+  gtk_table_attach(GTK_TABLE(table),label2,0,1,1,2, 0,0,0,0);
+
+  widget1=gck_entryfield_new(NULL,NULL,mapvals.material.ambient_int,(GtkSignalFunc)entry_update);
+  widget2=gck_entryfield_new(NULL,NULL,mapvals.material.diffuse_int,(GtkSignalFunc)entry_update);
+
+  gtk_table_attach(GTK_TABLE(table),widget1,2,3,0,1, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_table_attach(GTK_TABLE(table),widget2,2,3,1,2, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+
+  style=gtk_widget_get_style(table);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],amb1_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,1,2,0,1, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],amb2_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,3,4,0,1, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],diffint1_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,1,2,1,2, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],diffint2_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,3,4,1,2, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  gtk_widget_show(label1);
+  gtk_widget_show(label2);
+  gtk_widget_show(widget1);
+  gtk_widget_show(widget2);
+  gtk_widget_show(table);
+  gtk_widget_show(frame);
+
+  gtk_tooltips_set_tip(tooltips,widget1,"Amount of original color to show where no direct light falls",NULL);
+  gtk_tooltips_set_tip(tooltips,widget2,"Intensity of original color when lit by a light source",NULL);
 
   gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.material.ambient_int);
   gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.material.diffuse_int);
 
-  frame=gck_frame_new("Reflectivity",page3->widget,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
-  hbox=gck_hbox_new(frame,FALSE,TRUE,TRUE,0,0,5);
-  vbox=gck_vbox_new(hbox,TRUE,FALSE,TRUE,0,0,0);
-  frame=gck_frame_new(NULL,vbox,GTK_SHADOW_NONE,TRUE,TRUE,0,0);
-  gck_label_aligned_new("Diffuse:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
-  frame=gck_frame_new(NULL,vbox,GTK_SHADOW_NONE,TRUE,TRUE,0,0);
-  gck_label_aligned_new("Specular:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
-  frame=gck_frame_new(NULL,vbox,GTK_SHADOW_NONE,TRUE,TRUE,0,0);
-  gck_label_aligned_new("Hightlight:",frame,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
-  vbox=gck_vbox_new(hbox,FALSE,FALSE,FALSE,5,0,0);
+  frame=gck_frame_new("Reflectivity",page,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
 
-  widget1=gck_entryfield_new(NULL,vbox,mapvals.material.diffuse_ref,(GtkSignalFunc)entry_update);
-  widget2=gck_entryfield_new(NULL,vbox,mapvals.material.specular_ref,(GtkSignalFunc)entry_update);
-  widget3=gck_entryfield_new(NULL,vbox,mapvals.material.highlight,(GtkSignalFunc)entry_update);
+  table=gtk_table_new(3,4,FALSE);
+  gtk_container_add(GTK_CONTAINER(frame),table);
+  
+  label1=gck_label_aligned_new("Diffuse:",NULL,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED); 
+  label2=gck_label_aligned_new("Specular:",NULL,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
+  label3=gck_label_aligned_new("Hightlight:",NULL,GCK_ALIGN_RIGHT,GCK_ALIGN_CENTERED);
 
-  gtk_tooltips_set_tip (tooltips,widget1,"Higher values makes the surface reflect more light (appear lighter)",NULL);
-  gtk_tooltips_set_tip (tooltips,widget2,"Controls how intense the highlights will be",NULL);
-  gtk_tooltips_set_tip (tooltips,widget3,"Higher values makes the highlights more focused",NULL);
+  gtk_table_attach(GTK_TABLE(table),label1,0,1,0,1, 0,0,0,0);
+  gtk_table_attach(GTK_TABLE(table),label2,0,1,1,2, 0,0,0,0);
+  gtk_table_attach(GTK_TABLE(table),label3,0,1,2,3, 0,0,0,0);
+
+  widget1=gck_entryfield_new(NULL,NULL,mapvals.material.diffuse_ref,(GtkSignalFunc)entry_update);
+  widget2=gck_entryfield_new(NULL,NULL,mapvals.material.specular_ref,(GtkSignalFunc)entry_update);
+  widget3=gck_entryfield_new(NULL,NULL,mapvals.material.highlight,(GtkSignalFunc)entry_update);
+
+  gtk_table_attach(GTK_TABLE(table),widget1,2,3,0,1, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_table_attach(GTK_TABLE(table),widget2,2,3,1,2, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_table_attach(GTK_TABLE(table),widget3,2,3,2,3, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+
+  gtk_tooltips_set_tip(tooltips,widget1,"Higher values makes the object reflect more light (appear lighter)",NULL);
+  gtk_tooltips_set_tip(tooltips,widget2,"Controls how intense the highlights will be",NULL);
+  gtk_tooltips_set_tip(tooltips,widget3,"Higher values makes the highlights more focused",NULL);
 
   gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.material.diffuse_ref);
   gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.material.specular_ref);
   gtk_object_set_data(GTK_OBJECT(widget3),"ValuePtr",(gpointer)&mapvals.material.highlight);
 
+  style=gtk_widget_get_style(table);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],diffref1_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,1,2,0,1, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],diffref2_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,3,4,0,1, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],specref1_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,1,2,1,2, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],specref2_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,3,4,1,2, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],high1_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,1,2,2,3, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  image=gdk_pixmap_create_from_xpm_d(appwin->widget->window,&mask,&style->bg[GTK_STATE_NORMAL],high2_xpm);
+  pixmap=gtk_pixmap_new(image,mask);
+  gtk_table_attach(GTK_TABLE(table),pixmap,3,4,2,3, GTK_EXPAND|GTK_FILL,GTK_EXPAND|GTK_FILL, 0,0);
+  gtk_widget_show(pixmap);
+
+  gtk_widget_show(label1);
+  gtk_widget_show(label2);
+  gtk_widget_show(label3);
+  gtk_widget_show(widget1);
+  gtk_widget_show(widget2);
+  gtk_widget_show(widget3);
+  gtk_widget_show(table);
+  gtk_widget_show(frame);
+
+  gtk_widget_show(page);
+  
+  return page;
+}
+
+GtkWidget *create_bump_page(void)
+{
+  GtkWidget *page,*widget1,*widget2,*imagewid;
+  GtkWidget *frame,*vbox,*label;
+
   /* Bump mapping page */
   /* ================= */
   
-  bump_page=gck_notebook_page_new("Bump",options_notebook);
+  page=gtk_vbox_new(FALSE,0);
 
-  frame=gck_frame_new("Bumpmap settings",bump_page->widget,GTK_SHADOW_ETCHED_IN,TRUE,FALSE,0,5);
+  frame=gck_frame_new("Bumpmap settings",page,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
   vbox=gck_vbox_new(frame,FALSE,TRUE,TRUE,5,0,5);
-/*  widget1=gck_option_menu_new("Map type:",vbox,TRUE,TRUE,0,
-    maptype_labels,(GtkSignalFunc)mapmenu1_callback,NULL); */
 
   imagewid=gck_vbox_new(vbox,FALSE,TRUE,TRUE,5,0,5);
 
   widget1=gck_hbox_new(imagewid,FALSE,TRUE,TRUE,5,0,3);
-  gck_label_new("Bumpmap image:",widget1);
+  label = gck_label_new("Bumpmap image:",widget1);
+
+  gtk_widget_show(label);
+  gtk_widget_show(widget1);
 
   widget2=gtk_option_menu_new();
   gtk_box_pack_end(GTK_BOX(widget1),widget2,TRUE,TRUE,0);
+  gtk_widget_show(widget2);
 
   widget1 = gimp_drawable_menu_new (bumpmap_constrain, bumpmap_drawable_callback,
     NULL, mapvals.bumpmap_id);
   gtk_option_menu_set_menu(GTK_OPTION_MENU(widget2), widget1);
 
-  gtk_widget_show(widget2);
-
+  gck_auto_show(TRUE);
   widget1=gck_option_menu_new("Curve:",imagewid,TRUE,TRUE,0,curvetype_labels,
     (GtkSignalFunc)mapmenu2_callback,NULL);
   gtk_option_menu_set_history(GTK_OPTION_MENU(widget1),mapvals.bumpmaptype);
+  gck_auto_show(FALSE);
 
   widget1=gck_entryfield_new("Minimum height:",imagewid,mapvals.bumpmin,
     (GtkSignalFunc)entry_update);
@@ -707,43 +904,34 @@ void create_main_notebook(GtkWidget *container)
 
   gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.bumpmin);
   gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.bumpmax);
+  gtk_widget_show(widget1);
+  gtk_widget_show(widget2);
+
   widget1=gck_checkbutton_new("Autostretch to fit value range",imagewid,mapvals.bumpstretch,
     (GtkSignalFunc)togglestretch_update);
   gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.bumpstretch);
+  gtk_widget_show(widget1);
 
-/*  waveswid=gck_vbox_new(vbox,FALSE,TRUE,TRUE,2,0,2);
-  widget1=gck_frame_new("Center",waveswid,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,2);
-  vbox2=gck_vbox_new(widget1,FALSE,TRUE,TRUE,2,0,2);
-  widget1=gck_entryfield_new("X:",vbox2,mapvals.wave_cx,(GtkSignalFunc)entry_update);
-  widget2=gck_entryfield_new("Y:",vbox2,mapvals.wave_cy,(GtkSignalFunc)entry_update);
+  gtk_widget_show(imagewid);
+  gtk_widget_show(vbox);
+  gtk_widget_show(frame);
+  gtk_widget_show(page);
 
-  gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.wave_cx);
-  gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.wave_cy);
+  return(page);
+}
 
-  widget1=gck_frame_new("Wavelength",waveswid,GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,2);
-  vbox2=gck_vbox_new(widget1,FALSE,TRUE,TRUE,2,0,2);
-  widget1=gck_entryfield_new("X:",vbox2,mapvals.wave_lx,(GtkSignalFunc)entry_update);
-  widget2=gck_entryfield_new("Y:",vbox2,mapvals.wave_ly,(GtkSignalFunc)entry_update);
-  
-  gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.wave_lx);
-  gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.wave_ly);
-
-  widget1=gck_entryfield_new("Amplitude:",waveswid,mapvals.wave_amp,(GtkSignalFunc)entry_update);
-  widget2=gck_entryfield_new("Phase:",waveswid,mapvals.wave_ph,(GtkSignalFunc)entry_update);
-
-  gtk_object_set_data(GTK_OBJECT(widget1),"ValuePtr",(gpointer)&mapvals.wave_amp);
-  gtk_object_set_data(GTK_OBJECT(widget2),"ValuePtr",(gpointer)&mapvals.wave_ph);
-
-  gtk_widget_hide(waveswid); */
-  gtk_widget_hide(bump_page->widget);
+GtkWidget *create_environment_page(void)
+{
+  GtkWidget *page,*frame,*vbox,*imagewid;
+  GtkWidget *widget1,*widget2;
 
   /* Environment mapping page */
   /* ======================== */
   
-  env_page=gck_notebook_page_new("Env",options_notebook);
+  page=gck_vbox_new(NULL,FALSE,FALSE,FALSE,0,0,0);
 
-  frame=gck_frame_new("Environment settings",env_page->widget,
-    GTK_SHADOW_ETCHED_IN,TRUE,FALSE,0,5);
+  frame=gck_frame_new("Environment settings",page,
+    GTK_SHADOW_ETCHED_IN,FALSE,FALSE,0,5);
   vbox=gck_vbox_new(frame,FALSE,TRUE,TRUE,5,0,5);
 
   imagewid=gck_vbox_new(vbox,FALSE,TRUE,TRUE,5,0,5);
@@ -754,34 +942,73 @@ void create_main_notebook(GtkWidget *container)
   widget2=gtk_option_menu_new();
   gtk_box_pack_end(GTK_BOX(widget1),widget2,TRUE,TRUE,0);
 
+  gtk_widget_show(widget1);
+
   widget1 = gimp_drawable_menu_new (envmap_constrain, envmap_drawable_callback,
     NULL, mapvals.envmap_id);
   gtk_option_menu_set_menu(GTK_OPTION_MENU(widget2), widget1);
 
   gtk_widget_show(widget2);
+  gtk_widget_show(imagewid);
+  gtk_widget_show(vbox);
+  gtk_widget_show(frame);
+  gtk_widget_show(page);
 
-  gtk_widget_hide(env_page->widget);
+  return(page);  
+}
 
-  /* Refraction mapping page */
-  /* ======================= */
+/****************************/
+/* Create notbook and pages */
+/****************************/
 
-  ref_page=gck_notebook_page_new("Refraction",options_notebook);
+void create_main_notebook(GtkWidget *container)
+{
+  GtkWidget *page,*label;
 
-  /* Insert the pages into the notebook */
-  /* ================================== */
+  gck_auto_show(FALSE);
 
-  gck_notebook_append_page(options_notebook,page1);
-  gck_notebook_append_page(options_notebook,page2);
-  gck_notebook_append_page(options_notebook,page3);
+  options_note_book=GTK_NOTEBOOK(gtk_notebook_new());
+  gtk_container_add(GTK_CONTAINER(container),GTK_WIDGET(options_note_book));
+
+  page = create_options_page();
+  label=gtk_label_new("Options");
+  gtk_widget_show(label);
+
+  gtk_notebook_append_page(options_note_book,page,label);
+  
+  page = create_light_page();
+  label=gtk_label_new("Light");
+  gtk_widget_show(label);
+
+  gtk_notebook_append_page(options_note_book,page,label);
+  
+  page = create_material_page();
+  label=gtk_label_new("Material");
+  gtk_widget_show(label);
+
+  gtk_notebook_append_page(options_note_book,page,label);
 
   if (mapvals.bump_mapped==TRUE)
-    gck_notebook_append_page(options_notebook,bump_page);
+    {
+      bump_page = create_bump_page();
+      bump_label=gtk_label_new("Bump");
+      gtk_widget_show(bump_label);
+      bump_page_pos = g_list_length(options_note_book->children);
+      gtk_notebook_append_page(options_note_book,bump_page,bump_label);      
+    }
 
   if (mapvals.env_mapped==TRUE)
-    gck_notebook_append_page(options_notebook,env_page);
+    {
+      env_page = create_environment_page();
+      env_label=gtk_label_new("Env");
+      gtk_widget_show(env_label);
+      env_page_pos = g_list_length(options_note_book->children);
+      gtk_notebook_append_page(options_note_book,env_page,env_label);
+    }
   
-  if (mapvals.ref_mapped==TRUE)
-    gck_notebook_append_page(options_notebook,ref_page);
+  gtk_widget_show(GTK_WIDGET(options_note_book));
+
+  gck_auto_show(TRUE);
 }
 
 /*****************************************************/
@@ -795,6 +1022,7 @@ void create_main_dialog(void)
   GtkWidget *frame,*applybutton,*cancelbutton,*helpbutton,*hbox,*wid;
 
   appwin = gck_application_window_new("Lighting effects");
+  gtk_widget_realize(appwin->widget);
 
   tooltips=gtk_tooltips_new();
 
@@ -828,8 +1056,8 @@ void create_main_dialog(void)
   gtk_widget_grab_default (applybutton);
   gtk_widget_set_sensitive(helpbutton,FALSE);
   
-  gtk_tooltips_set_tip (tooltips,applybutton,"Apply filter with current settings",NULL);
-  gtk_tooltips_set_tip (tooltips,cancelbutton,"Close filter without doing anything",NULL);
+  gtk_tooltips_set_tip(tooltips,applybutton,"Apply filter with current settings",NULL);
+  gtk_tooltips_set_tip(tooltips,cancelbutton,"Close filter without doing anything",NULL);
 
   /* Split the workarea in two */
   /* ========================= */
@@ -850,12 +1078,12 @@ void create_main_dialog(void)
 
   hbox=gck_hbox_new(workbox1b,FALSE,FALSE,FALSE,5,0,0);
   wid=gck_pushbutton_new("  Preview!  ",hbox,FALSE,FALSE,0,(GtkSignalFunc)preview_callback);
-  gtk_tooltips_set_tip (tooltips,wid,"Recompute preview image",NULL);
+  gtk_tooltips_set_tip(tooltips,wid,"Recompute preview image",NULL);
   hbox=gck_hbox_new(hbox,TRUE,FALSE,FALSE,0,0,0);
   wid=gck_pushbutton_new(" + ",hbox,FALSE,FALSE,0,(GtkSignalFunc)zoomin_callback);
-  gtk_tooltips_set_tip (tooltips,wid,"Zoom in (make image bigger)",NULL);
+  gtk_tooltips_set_tip(tooltips,wid,"Zoom in (make image bigger)",NULL);
   wid=gck_pushbutton_new(" - ",hbox,FALSE,FALSE,0,(GtkSignalFunc)zoomout_callback);
-  gtk_tooltips_set_tip (tooltips,wid,"Zoom out (make image smaller)",NULL);
+  gtk_tooltips_set_tip(tooltips,wid,"Zoom out (make image smaller)",NULL);
 
   create_main_notebook(workbox2);
 
